Secure.ASM
; Program..: Secure.ASM
; Author...: Roger Wegehoft 
; Date.....: June 1, 1988
; Notes....: Data Encryption/Decryption "Load And Call" Procedure
;            for dBASE III PLUS.
;
;            Use: Declare mkey and mdata in consecutive order since 
;                 mkey will be used to locate mdata.
;
;            STORE "<key>  ..." to mkey
;            STORE "<data> ..." to mdata
;
;            Syntax: CALL Secure WITH mkey
;
;*******************************************************************************
; Structure Definitions
; (Storage declared at end of CODE segment)
;
MEMVAR	STRUC		; Structure of a dBASE memvar.
	LEN	DB ?	; Length of memvar.
	LOC	DW ?	; Offset from start of data segment.
MEMVAR	ENDS 
LOCVAR	STRUC		; Group of local index variables.
	J	DB ?	; Key memvar position index.
	I	DB ?	; Data memvar position index.
	IPT	DB ?	; Index plain text storage.
LOCVAR	ENDS
;******************************************************************************* 
;
CODE	SEGMENT PAGE
;===============================================================================
SECURE	PROC	FAR
	ASSUME	CS:CODE
START:
	PUSH	AX	; Save registers.
	PUSH	BX	;
	PUSH	CX	;
	PUSH	DX	;
	PUSH	SI	;
	PUSH	DI	;
	PUSH	DS	;
	MOV	CS:CKEY.LOC,BX	; Save location of key memvar.
	MOV	SI,BX	; Set key index pointer.
	MOV	AL,DS:[BX-1]	; Get the length of the key memvar.
	DEC	AL	; Don't want the null to count.
	MOV	CS:CKEY.LEN,AL	; Save it. 
	XOR	AH,AH	; Zero high order byte
	ADD	AX,2	; Point to next memvar. 
	ADD	BX,AX	; Compute offset to data memvar.
	MOV	CS:CINFO.LOC,BX	; Save offset to data mvar.
	MOV	DI,BX	; Set data index pointer.
	MOV	AL,DS:[BX-1]	; Get the length of the memvar.
	DEC	AL	; Don't want the null to count.
	MOV	CS:CINFO.LEN,AL	; Save it.
	MOV	BX,CS:CKEY.LOC	; Reinitilize base pointer.
	XOR	AL,AL	; Zero AL to set loop counters. 
	MOV	CS:LVARS.I,AL	; Set loop counters.
	MOV	CS:LVARS.J,AL	; Set loop counters.
	MOV	CL,CS:LVARS.I	; Initialize loop register.
	JMP	CHECKLEN
LOOPER:
	ADD	CL,1	; Bump loop counter.
CHECKLEN:
	CMP	CL,CS:CINFO.LEN	; Test for end of data. 
	JL	ENCRYPT	; More data.
	JMP	DONE	; Done.
ENCRYPT:
	MOV	AL,DS:[DI]	; Get data byte.
	SUB	AL,32	; Compute index into set.
	MOV	CS:LVARS.IPT,AL	; Store IPT.
	MOV	AL,DS:[SI]	; Get key element.
	SUB	AL,32	; Compute index into set.
	XOR	AL,CS:LVARS.IPT	; Encrypt data element with key element.
	ADD	AL,32	; Translate back to data set range.
	MOV	DS:[DI],AL	; Put data back in memvar.
	INC	DI	; Bump data index.
	INC	SI	; Bump key index.
	MOV	AL,CS:LVARS.J	; Get the key index counter.
	INC	AL	; Bump it.
	MOV	CS:LVARS.J,AL	; Put it back.
	CMP	AL,CS:CKEY.LEN	; Test it. 
	JGE	RESET	; Reset to beginning of key.
	JMP	NORESET	; Skip around key-reset.
RESET:
	MOV	AL,1	; Start of key.
	MOV	CS:LVARS.J,AL	; Store it.
	MOV	SI,CS:CKEY.LOC	; Reset key index pointer.
NORESET:
	JMP	LOOPER	; Next pass.
DONE:
	POP	DS	; Restore registers.
	POP	DI	;
	POP	SI	;
	POP	DX	;
	POP	CX	;
	POP	BX	;
	POP	AX	;
	RET		; Far return to dBASE.
;===============================================================================
; CODE Segment Data.
;
LVARS	LOCVAR   <>	; Storage for local variables.
CKEY	MEMVAR   <>	; Storage for key data.
CINFO	MEMVAR   <>	; Storage for data to process.
SECURE	ENDP
CODE	ENDS
	END      START

